var express = require('express');
var path = require('path');
var logger = require('morgan');
var cookieParser = require('cookie-parser');
var bodyParser = require('body-parser');
var http = require('http');
var debug = require('debug')('axon-web-server:server');
var session = require('express-session');
var webpack = require("webpack");
var webpackDevMiddleware = require("webpack-dev-middleware");
var multer = require('multer');
var storage = multer.memoryStorage();
var upload = multer({storage: storage});
var pkg = require('./package.json');

exports.start = function (router, config = {}) {
  var app = express();
  var server = http.createServer(app);

  var routes = require('./routes/index');
  var api = require('./routes/api');
  var login = require('./routes/login');

  var exclude = function (paths = [], middleware) {
    return function (req, res, next) {
      return paths.indexOf(req.path) != -1 ? next() : middleware(req, res, next);
    };
  };

  const port = config.port || 3000;

  app.set('views', path.join(__dirname, 'views'));
  app.set('view engine', 'jade');
  app.set('port', parseInt(port, 10));

  app.use(logger('dev'));
  app.use(bodyParser.json());
  app.use(bodyParser.urlencoded({extended: false}));
  app.use(cookieParser());

  app.use(function (req, res, next) {
    req.environment = config.environment;
    next();
  });

  if (app.get('env') == 'development' && config.webpack) {
    app.use(webpackDevMiddleware(webpack(config.webpack.compile), config.webpack.middleware));
  }

  if (config.static) {
    app.use(express.static(config.static));
  }

  app.all('/file/*', upload.any());

  if (config.session) {
    app.use(session(config.session));
    app.use('/login', login);
    app.use(exclude(config.exclude, function (req, res, next) {
      req.session.user ? next() : res.redirect('/login');
    }));
  }

  app.use(router);
  app.use('/api', api);
  app.use('/', routes);

  app.all('*', config.closingMiddleware || ((req, res) => {
    res.render('index', {
      font: config.font || 'https://fonts.googleapis.com/css?family=Open+Sans:400,600,700'
    });
  }));

  // development error handler
  // will print stacktrace
  if (app.get('env') === 'development') {
    app.use(function (err, req, res, next) {
      res.status(err.status || 500);
      res.render('error', {
        message: err.message,
        error: err
      });
    });
  }

  // production error handler
  // no stacktraces leaked to user
  app.use(function (err, req, res, next) {
    res.status(err.status || 500);
    res.render('error', {
      message: err.message,
      error: {}
    });
  });

  server.listen(port);
  server.on('error', onError);
  server.on('listening', onListening);

  function onListening() {
    var addr = server.address();
    var bind = typeof addr === 'string'
      ? 'pipe ' + addr
      : 'port ' + addr.port;
    debug('Listening on ' + bind);
  }

  function onError(error) {
    if (error.syscall !== 'listen') {
      throw error;
    }

    var bind = typeof port === 'string'
      ? 'Pipe ' + port
      : 'Port ' + port;

    // handle specific listen errors with friendly messages
    switch (error.code) {
      case 'EACCES':
        console.error(bind + ' requires elevated privileges');
        process.exit(1);
        break;
      case 'EADDRINUSE':
        console.error(bind + ' is already in use');
        process.exit(1);
        break;
      default:
        throw error;
    }
  }
};